/*
 *  Viewer.h
 *
 *  The Viewer class used to create new specific viewers
 *
 *  Copyright (C) 2002 Tobias Dyckerhoff
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef VIEWER_H
#define VIEWER_H
 
#include <iostream>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/entry.h>
#include <gtk--/main.h>
#include <gtk--/window.h>
#include <gtk--/table.h>
#include <gtk--/label.h>
#include <gtk--/separator.h>
#include <gtk--/toolbar.h>
#include <gtkgl--/glarea.h>
#include <gtk--/frame.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include <math.h>

#include "QuatMath.h"  
#include "Vector3.h"  
#include "SpaceEntity.h"
#include "widgets/Gtk_CameraControl.h"
#include "widgets/Gtk_ViewerToolbox.h"

class Viewer;



/** Basic OpenGL viewing window.
    The Viewer consists of an GLArea object nested within a single window.
    Mouse events like clicking and moving are handled within appropiate methods,
    which enables the user to rotate the scene and zoom in and out.
    No content is drawn yet. The Viewer is meant to inherit to another Class 
    where then GL content can be specified by overriding the virtual method 
    draw_gl_context_imp. The mouse handling methods can also be overriden. 
    Finally there are public Gtk container members to insert additional widgets
    afterwards.
*/

class Viewer : public Gtk::Window
{
  Gtk::HBox hbox;
  Gtk::Frame glframe;

  Gtk::VSeparator sep1;
  Gtk::VSeparator sep2;
  Gtk::VSeparator sep3;
  Gtk::VSeparator sep4;


  Gtk::VBox vbox;

  gint delete_event_impl(GdkEventAny*) {
      Gtk::Main::quit(); 
      return 0;
  }


  /* some variables for the mouse control */

  /* the following should be protected
     ( allow set_usize(), similar mouse_handling,... to derived classes) */
 protected:

  gdouble gl_mouse_start_x, gl_mouse_start_y;
  gboolean gl_mouse1_pressed; 
  gboolean gl_mouse2_pressed;
  gboolean gl_mouse3_pressed;

  double gl_rot_x;
  double gl_rot_y;
  double gl_rot_z;
  double gl_zoom;


 public:

  /** The GLArea widget */
  Gtk::GLArea display;

  /** Constructor.
      Argument is a string which labels the Viewer window. */
  Viewer(const char* name);      

  /** This method initializes the Viewer specific GL environment. It is called 
      within the constructor. Right now there is no need to call this method 
      otherwise. */
  void initgl(); 

  /** Updates all the gl stats like gluLookat. It is called whenever the camera 
      stats have changed, for example. */
  void updategl(); 

  /** Draws the GL content. Calls draw_gl_context_imp. Whenever the content or 
      the camera stats have changed, drawgl is called. */
  gint drawgl(); 

  /** Virtual method which is called within initgl. It can be overriden to 
      initialize some gl content like display lists. */
  virtual gint init_gl_context_imp();

  /** Within this virtual method all drawing calls are processed. The GL content
      is specified here. drawgl must be called to do the actual drawing! */
  virtual gint draw_gl_context_imp();

  /** This handling method is called when ever an expose event occurs, eg when
      the mouse enters the window (?) */
  gint gl_expose_handler(GdkEventExpose* expose);

  /** Another handling method. It is called when the window is resized. */
  gint gl_configure_handler(GdkEventConfigure* configure);

  /** Virtual method which handels mouse motion. Contains the standard Viewer 
      mouse control. Override to realize alternative mouse control.
  */
  virtual gint gl_motion_notify_handler_imp(GdkEventMotion* event); 

  /** Virtual method which handels mouse clicks. Contains the standard Viewer 
      mouse control. Override to realize alternative mouse control. */
  virtual gint gl_button_press_handler_imp(GdkEventButton* event);

  /** Virtual method which handels mouse clicks. Contains the standard Viewer 
      mouse control. Override to realize alternative mouse control. */
  virtual gint gl_button_release_handler_imp(GdkEventButton* event); 

  /** This is the object which specifies position, lookat and up orientation 
      of the GL camera (via a gluLookat call within updategl).  */
  SpaceEntity camera;


  /** Specifies the light position. */
  Vector3 gl_light_pos;

  /** Change the light position. */
  int set_light(Vector3 pos);

  /** One of two containers in which additional widgets can be inserted 
      afterwards. This one is located above the GL widget. */
  Gtk::HBox upper_hbox;

 /** One of two containers in which additional widgets can be inserted 
     afterwards. This one is located beneath the GL widget. */
  Gtk::HBox lower_hbox;
 
   
 /** This signal will be emmitted whenever the gl stats have changed. It can be 
     used to monitor the camera stats in other objects. */
  SigC::Signal0<void> gl_changed_event;

};


#endif














